//+------------------------------------------------------------------+
//|                                        wpr + bollinger bands.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DeepSkyBlue
#property indicator_color2  Snow
#property indicator_color3  DarkOrange
#property indicator_color4  Snow
#property indicator_style2  STYLE_DASH
#property indicator_style3  STYLE_DASH
#property indicator_style4  STYLE_DASH
#property indicator_width1  4
#property indicator_maximum 120
#property indicator_minimum -20
#property indicator_level1  20
#property indicator_level2  80
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor Magenta

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    HLR_Range       = 100;
extern int    BandsPeriod     = 20;
extern double BandsDeviations = 3.2;  
extern bool   Interpolate     = true;

//
//
//
//
//

double HLR[];
double bandsUp[];
double bandsMi[];
double bandsDo[];

string IndicatorFileName;
bool   CalculatingHLR;
bool   ReturningBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,HLR);
   SetIndexBuffer(1,bandsUp);
   SetIndexBuffer(2,bandsMi);
   SetIndexBuffer(3,bandsDo);

   //
   //
   //
   //
   //
   
      IndicatorFileName   = WindowExpertName();
      CalculatingHLR      = (TimeFrame=="CalculateHLR"); if (CalculatingHLR) return(0);
      ReturningBars       = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame           = stringToTimeFrame(TimeFrame);
         
   //
   //
   //
   //
   //
               
   IndicatorShortName("Range osc with bands "+timeFrameToString(timeFrame)+" ("+HLR_Range+","+BandsPeriod+","+DoubleToStr(BandsDeviations,2)+")");
   return(0);
}
int deinit() { return(0); }




int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { HLR[0] = limit; return(0); }

   
   
      if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      if (CalculatingHLR)
      {
          
      for(i=limit; i >= 0; i--)
      {
      double hhv = iHigh(NULL,0,iHighest(NULL,0,MODE_HIGH,HLR_Range,i));
      double llv = iLow(NULL,0,iLowest(NULL,0,MODE_LOW,HLR_Range,i)); 
      double mpr = (iHigh(NULL,0,i) + iLow(NULL,0,i)) / 2.0; 
      if(hhv!=llv)
      HLR[i] = 100.0 * (mpr-llv)/(hhv-llv); 
      else HLR[i] = 0.0; 
      }
      for(i=limit; i>=0; i--) 
      {
      double deviations = iStdDevOnArray(HLR,0,BandsPeriod,0,MODE_LWMA,i);
      bandsMi[i] = iMAOnArray(HLR,0,BandsPeriod,0,MODE_LWMA,i);
      bandsUp[i] = bandsMi[i]+deviations*BandsDeviations;
      bandsDo[i] = bandsMi[i]-deviations*BandsDeviations;
         
     }
     return(0);
   }

   

      for(i=limit; i>=0; i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
            HLR[i]     = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,BandsPeriod,BandsDeviations,0,y);
            bandsUp[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,BandsPeriod,BandsDeviations,1,y);
            bandsMi[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,BandsPeriod,BandsDeviations,2,y);
            bandsDo[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateHLR",HLR_Range,BandsPeriod,BandsDeviations,3,y);

         
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         

         datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int k = 1; k < n; k++)
         {
  	      HLR[i+k]     = k*factor*HLR[i+n]     + (1.0-k*factor)*HLR[i];
  	      bandsUp[i+k] = k*factor*bandsUp[i+n] + (1.0-k*factor)*bandsUp[i];
  	      bandsMi[i+k] = k*factor*bandsMi[i+n] + (1.0-k*factor)*bandsMi[i];
  	      bandsDo[i+k] = k*factor*bandsDo[i+n] + (1.0-k*factor)*bandsDo[i];
        }  	            
   }
      
   
      
   return(0);
}




string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}